@@ -1,16 +1,18 @@
 """tapdance.cli - Defines the command-line interface (CLI) for tapdance."""
 
 import fire
+from logless import logged
 
+from tapdance.config import print_version
 from tapdance.docker import build_all_images, build_image
 from tapdance.install_helper import install
 from tapdance.plans import plan
 from tapdance.syncs import sync
 
 
+@logged("tapdance execution")
 def main():
     """Run the tapdance CLI."""
-    print_version()
     fire.Fire(
         {
             "plan": plan,
@@ -18,24 +20,10 @@ def main():
             "install": install,
             "build_all_images": build_all_images,
             "build_image": build_image,
-            "version": print_version,
+            "--version": print_version,
         }
     )
 
 
-def print_version():
-    """Print the tapdance version number."""
-    try:
-        from importlib import metadata
-    except ImportError:
-        # Running on pre-3.8 Python; use importlib-metadata package
-        import importlib_metadata as metadata
-    try:
-        version = metadata.version("tapdance")
-    except metadata.PackageNotFoundError:
-        version = "[could not be detected]"
-    print(f"tapdance version {version}")
-
-
 if __name__ == "__main__":
     main()
