@@ -37,13 +37,16 @@ def _is_valid_json(json_text: str):
 @logged("running discovery on '{tap_name}'")
 def _discover(
     tap_name: str,
+    taps_dir: str,
     *,
     config_file: str,
     catalog_dir: str,
     dockerized: bool,
     tap_exe: str,
 ) -> None:
-    catalog_file = config.get_raw_catalog_file(catalog_dir, tap_name)
+    catalog_file = config.get_raw_catalog_file(
+        taps_dir, catalog_dir, tap_name, allow_custom=False
+    )
     uio.create_folder(catalog_dir)
     img = f"{docker.BASE_DOCKER_REPO}:{tap_exe}"
     hide_cmd = False
@@ -60,7 +63,7 @@ def _discover(
                 hide_cmd = True
         _, _ = runnow.run(f"docker pull {img}")
         _, output_text = runnow.run(
-            f"docker run --rm "
+            f"docker run --rm -i "
             f"-v {cdw}:/home/local {tap_docker_args} "
             f"{img} --config {config.dockerize_cli_args(config_file)} --discover",
             echo=False,
@@ -77,30 +80,149 @@ def _discover(
         )
 
 
+@logged("running custom schema inference on a dry run of '{tap_name}'")
+def _infer_schema(
+    tap_name: str,
+    taps_dir: str,
+    raw_catalog_file: str,
+    selected_catalog_file: str,
+    *,
+    config_file: str,
+    catalog_dir: str,
+    dockerized: bool,
+    tap_exe: str,
+) -> str:
+    custom_catalog = json.loads(uio.get_text_file_contents(raw_catalog_file))
+    tmp_folder = f"{catalog_dir}/tmp"
+    tmp_outfile = f"{catalog_dir}/tmp/sync-dryrun.jsonl"
+    uio.create_folder(catalog_dir)
+    uio.create_folder(tmp_folder)
+    logging.info(f"Cleaning up old files in tmp folder '{tmp_folder}'...")
+    for file in uio.list_files(tmp_folder):
+        if any(
+            [
+                file.endswith(x)
+                for x in ["-config.json", "-dryrun.jsonl", "-table.inferred.json"]
+            ]
+        ):
+            uio.delete_file(file)
+    img = f"{docker.BASE_DOCKER_REPO}:{tap_exe}"
+    hide_cmd = False
+    if dockerized:
+        cdw = os.getcwd().replace("\\", "/")
+        tap_config = json.loads(uio.get_text_file_contents(config_file))
+        tap_docker_args = ""
+        # TODO: Replace with logic to parse from AWS_SHARED_CREDENTIALS_FILE env var:
+        for k in ["aws_access_key_id", "aws_secret_access_key", "aws_session_token"]:
+            if k in tap_config:
+                key = f"TAP_{tap_name}_{k}".replace("-", "_").upper()
+                os.environ[key] = tap_config[k]
+                tap_docker_args += f' -e {k.upper()}="{tap_config[k]}"'
+                hide_cmd = True
+        _, _ = runnow.run(f"docker pull {img}")
+        _, jsonl_out = runnow.run(
+            f"docker run --rm -i "
+            f"-v {cdw}:/home/local {tap_docker_args} "
+            f"{img} "
+            f"--config {config.dockerize_cli_args(config_file)}"
+            f"--catalog {selected_catalog_file}",
+            hide=hide_cmd,
+            echo=False,
+            capture_stderr=False,
+        )
+    else:
+        _, jsonl_out = runnow.run(
+            f"{tap_exe} "
+            f"--config {config_file} "
+            f"--catalog {selected_catalog_file}",
+            hide=hide_cmd,
+            echo=False,
+            capture_stderr=False,
+        )
+    uio.create_text_file(tmp_outfile, jsonl_out)
+    _, jsonl_out = runnow.run(
+        f"cat {tmp_outfile} | singer-infer-schema --out-dir {tmp_folder}",
+    )
+    for file in uio.list_files(tmp_folder):
+        if file.endswith(".inferred.json"):
+            logging.info(f"Parsing inferred schema from '{file}'...")
+            inferred_schema = json.loads(uio.get_text_file_contents(file))
+            stream_name = file.split("/")[-1].split(".")[0]
+            stream = (
+                [x for x in custom_catalog["streams"] if x["stream"] == stream_name]
+                or [None]
+            )[0]
+            if not stream:
+                raise ValueError(
+                    f"Failed to append inferred schema for stream name '{stream_name}'."
+                    f" Stream not present in catalog file {selected_catalog_file}."
+                )
+            stream["schema"] = inferred_schema
+    custom_catalog_file = config.get_custom_catalog_file(taps_dir, tap_name)
+    uio.create_text_file(custom_catalog_file, json.dumps(custom_catalog, indent=2))
+    return custom_catalog_file
+
+
 def _check_rules(
-    catalog_file: str, rules_file: List[str]
-) -> Tuple[Dict[str, Dict[str, bool]], List[str]]:
-    """Evaluate rules against the contents of a catalog file.
+    tap_name: str,
+    catalog_file: str,
+    rules_file: str,
+    replication_strategy: str,
+    plan_file_out: str,
+    selected_catalog_file_out: str,
+    log_dir: Optional[str],
+) -> None:
+    """
+    Create plan file and selected catalog file from provided rules and raw catalog.
 
     Parameters
     ----------
     catalog_file : str
         Path to a catalog file.
-    rules_file : List[str]
+    rules_file : str
         Path to a rules file.
-
-    Returns
-    -------
-    Tuple[Dict[str, Dict[str, bool]], List[str]]
-        - Dictionary of tables names to dictionary of column names having values of True
-          (selected) or False (ignored)
-        - List of excluded tables
+    plan_file_out : str
+        Path to save the plan file.
+    selected_catalog_file_out : str
+        Path to save the selected catalog file.
     """
     select_rules = [
         line.split("#")[0].rstrip()
         for line in uio.get_text_file_contents(rules_file).splitlines()
         if line.split("#")[0].rstrip()
     ]
+    matches: Dict[str, dict] = {}
+    excluded_table_stream_ids: Dict[str, List[str]] = {}
+    matched_stream_ids: Dict[str, str] = {}
+    for stream_id, table_object in _get_catalog_tables_dict(catalog_file).items():
+        table_name = _get_stream_name(table_object)
+        if _table_match_check(
+            table_name=table_name, stream_id=stream_id, select_rules=select_rules,
+        ):
+            if table_name in matched_stream_ids:
+                raise RuntimeError(
+                    f"Table name '{table_name}' matched multiple stream IDs: "
+                    f'"{matched_stream_ids[table_name]}" and "{stream_id}". '
+                    "This is most often caused by tables with the same name under "
+                    "different source database schemas. Please qualify or disqualify "
+                    "specific stream name patterns by using double-quoted stream IDs "
+                    "in your rules file instead of or in addition to bare table names."
+                )
+            matched_stream_ids[table_name] = stream_id
+            matches[table_name] = {}
+            for col_object in _get_catalog_table_columns(table_object):
+                col_name = col_object
+                matches[table_name][col_name] = _col_match_check(
+                    table_name, stream_id, col_name, select_rules
+                )
+        else:
+            if table_name in excluded_table_stream_ids:
+                excluded_table_stream_ids[table_name].append(stream_id)
+            else:
+                excluded_table_stream_ids[table_name] = [stream_id]
+    all_matches_lower = [m.lower() for m in matches.keys()] + [
+        f'"{m.lower()}"' for m in matched_stream_ids.values()
+    ]
     declared_tables = set(
         [
             rule.split(".")[0].rstrip().lstrip("!")
@@ -108,33 +230,41 @@ def _check_rules(
             if rule.split(".")[0].rstrip() and ("*" not in rule.split(".")[0])
         ]
     )
-    matches: Dict[str, dict] = {}
-    excluded_table_list = []
-    for table_name, table_object in _get_catalog_tables_dict(catalog_file).items():
-        table_match_text = f"{table_name}"
-        if _table_match_check(table_match_text, select_rules):
-            matches[table_name] = {}
-            for col_object in _get_catalog_table_columns(table_object):
-                col_name = col_object
-                col_match_text = f"{table_name}.{col_name}"
-                matches[table_name][col_name] = _col_match_check(
-                    col_match_text, select_rules
-                )
-        else:
-            excluded_table_list.append(table_name)
     for required_table in declared_tables:
-        if required_table not in matches.keys():
+        if required_table.lower() not in all_matches_lower:
             logging.warning(
-                f"The table {required_table} was declared in the rules file "
+                f"The table '{required_table}' was declared in the rules file "
                 "but could not be found in the catalog."
             )
     for match, match_cols in matches.items():
         if not match_cols:
             logging.warning(
-                f"The table {match} was declared in the rules file "
+                f"The table '{match}' was declared in the rules file "
                 "but did not match with any columns in the catalog."
             )
-    return matches, excluded_table_list
+    primary_keys, replication_keys = _get_table_keys(
+        matches, matched_stream_ids, catalog_file, rules_file
+    )
+    file_text = _make_plan_file_text(
+        matches,
+        primary_keys,
+        replication_keys,
+        matched_stream_ids,
+        excluded_table_stream_ids,
+    )
+    logging.info(f"Updating plan file: {plan_file_out}")
+    uio.create_text_file(plan_file_out, file_text)
+    config.push_logs(log_dir, [rules_file, plan_file_out])
+    _create_selected_catalog(
+        tap_name,
+        plan_file=plan_file_out,
+        raw_catalog_file=catalog_file,
+        output_file=selected_catalog_file_out,
+        replication_strategy=replication_strategy,
+        skip_senseless_validators=SKIP_SENSELESS_VALIDATORS,
+    )
+    config.push_logs(log_dir, [selected_catalog_file_out])
+    # return matches, excluded_table_list
 
 
 def _validate_keys(table_object: dict, key_type: str):
@@ -214,6 +344,7 @@ def _set_catalog_file_keys(table_object: dict, table_plan: dict):
 def _get_catalog_file_keys(
     key_type: str,
     matches: Dict[str, Dict[str, bool]],
+    matched_stream_ids: Dict[str, str],
     catalog_file: str,
     warn_if_missing: bool = False,
 ) -> Dict[str, List[str]]:
@@ -236,11 +367,14 @@ def _get_catalog_file_keys(
         Mapping of table names to each table's list of keys
     """
     result: Dict[str, List[str]] = {}
-    for table_name, table_object in sorted(
+    for stream_id, table_object in sorted(
         _get_catalog_tables_dict(catalog_file).items()
     ):
+        table_name = _get_stream_name(table_object)
         if table_name not in matches.keys():
             continue
+        if stream_id != matched_stream_ids[table_name]:
+            continue
         result[table_name] = _get_table_key_cols(
             key_type, table_object, warn_if_missing=warn_if_missing
         )
@@ -280,6 +414,42 @@ def _get_rules_file_keys(
     return result
 
 
+def _get_table_keys(
+    matches: Dict[str, Dict[str, bool]],
+    matched_stream_ids: Dict[str, str],
+    catalog_file: str,
+    rules_file: str,
+) -> Tuple[Dict[str, List[str]], Dict[str, List[str]]]:
+    primary_keys: Dict[str, List[str]] = {}
+    replication_keys: Dict[str, List[str]] = {}
+    stream_name_lookup = {v: k for k, v in matched_stream_ids.items()}
+    for key_type, collection in [
+        ("primary-key", primary_keys),
+        ("replication-key", replication_keys),
+    ]:
+        collection.update(
+            _get_catalog_file_keys(
+                key_type,
+                matches=matches,
+                matched_stream_ids=matched_stream_ids,
+                catalog_file=catalog_file,
+            )
+        )
+        key_overrides = _get_rules_file_keys(
+            key_type, matches=matches, rules_file=rules_file
+        )
+        for table, override in key_overrides.items():
+            if table.startswith('"'):
+                table = stream_name_lookup[table.lstrip('"').rstrip('"')]
+            collection[table] = override
+    return primary_keys, replication_keys
+
+
+@logged(
+    "getting table list using: filter={table_filter}, exclude={exclude_tables}, "
+    "and catalog={catalog_file}",
+    success_msg=None,
+)
 def get_table_list(
     table_filter: Optional[Union[str, List[str]]],
     exclude_tables: Optional[Union[str, List[str]]],
@@ -305,7 +475,12 @@ def get_table_list(
     if isinstance(table_filter, list):
         list_of_tables = table_filter
     elif table_filter is None or table_filter == "*":
-        list_of_tables = sorted(_get_catalog_tables_dict(catalog_file).keys())
+        list_of_tables = sorted(
+            [
+                _get_stream_name(x)
+                for x in _get_catalog_tables_dict(catalog_file).values()
+            ]
+        )
     elif table_filter[0] == "[":
         # Remove square brackets and split the result on commas
         list_of_tables = table_filter.replace("[", "").replace("]", "").split(",")
@@ -327,21 +502,26 @@ def plan(
     *,
     dockerized: bool = None,
     rescan: bool = None,
+    infer_custom_schema: bool = None,
     tap_exe: str = None,
     taps_dir: str = None,
     config_dir: str = None,
     config_file: str = None,
+    log_dir: str = None,
     replication_strategy: str = None,
 ) -> None:
     """Perform all actions necessary to prepare (plan) for a tap execution.
 
-     1. Scan (discover) the source system metadata (if catalog missing or `rescan=True`)
-     2. Apply filter rules from `rules_file` and create human-readable `plan.yml` file to
-        describe planned inclusions/exclusions.
-     3. Add primary-key and replication-key to the catalog.json file if specified in the
-        rules file.
-     4. Create a new `catalog-selected.json` file which applies the plan file and which
-        can be used by the tap to run data extractions.
+    1. Capture raw catalog schema using discover (if needed or if --rescan).
+    2. If it exists, use the 'custom' catalog file in place of the 'raw' catalog.
+    3. Create the plan file and 'selected' version of the raw schema using
+       `*.rules.txt`.
+        - Add primary-key and replication-key to the catalog.json file if specified in the
+          rules file.
+    4. If infer_custom_schema=true:
+        - Use the 'selected' catalog to execute a dry run for 'infer_custom_schema'.
+        - Create or update the 'custom' catalog file using inferred schema.
+        - Rebuild the plan file and rebuild the 'selected' catalog.
 
     Parameters:
     -----------
@@ -353,6 +533,8 @@ def plan(
         Specifies the tap executable, if different from `tap-{tap_name}`.
     rescan : {bool}
         True to force a rescan and replace existing metadata.
+    infer_custom_schema : {bool}
+        True to infer schema by performing a dry run data sync.
     taps_dir: {str}
         The directory containing the rules file.
         (Default=cwd)
@@ -363,6 +545,9 @@ def plan(
     config_file : {str}
         The location of the JSON config file which contains config for the specified
         plugin. (Default=f"${config_dir}/${plugin_name}-config.json")
+    log_dir : {str}
+        Optional. The location to publish logs and other artifacts. If omitted, no
+        extra publishing will be performed.
     replication_strategy : {str}
         One of "FULL_TABLE", "INCREMENTAL", or "LOG_BASED"; by default "INCREMENTAL" or
         a value is set in the TAP_{TAPNAME}_REPLICATION_STRATEGY environment variable.
@@ -389,59 +574,56 @@ def plan(
         "REPLICATION_STRATEGY", "INCREMENTAL"
     )
     config.validate_replication_strategy(replication_strategy)
-
-    # TODO: Resolve bug in Windows STDERR inclusion when emitting catalog json from
-    #       docker run
-    # if (dockerized is None) and (uio.is_windows() or uio.is_mac()):
-    #     dockerized = True
-    #     logging.info(
-    #         "The 'dockerized' argument is not set when running either Windows or OSX. "
-    #         "Defaulting to dockerized=True..."
-    #     )
-
-    catalog_dir = config.get_catalog_output_dir(tap_name, taps_dir)
-    raw_catalog_file = config.get_raw_catalog_file(catalog_dir, tap_name)
+    catalog_dir = config.get_tap_output_dir(tap_name, taps_dir)
+    log_dir = config.get_log_dir(log_dir)
+    raw_catalog_file = config.get_raw_catalog_file(
+        taps_dir, catalog_dir, tap_name, allow_custom=True
+    )
     selected_catalog_file = f"{catalog_dir}/{tap_name}-catalog-selected.json"
     plan_file = config.get_plan_file(tap_name, taps_dir, required=False)
-
     if rescan or not uio.file_exists(raw_catalog_file):
         # Run discover, if needed, to get catalog.json (raw)
         _discover(
             tap_name,
+            taps_dir,
             config_file=config_file,
             catalog_dir=catalog_dir,
             dockerized=dockerized,
             tap_exe=tap_exe,
         )
-
+    config.push_logs(log_dir, [raw_catalog_file])
+    logging.info(f"Using catalog file for initial plan: {raw_catalog_file}")
     rules_file = config.get_rules_file(taps_dir, tap_name)
-    matches, excluded_tables = _check_rules(
-        catalog_file=raw_catalog_file, rules_file=rules_file
-    )
-    primary_keys = _get_catalog_file_keys(
-        "primary-key", matches=matches, catalog_file=raw_catalog_file
-    )
-    primary_keys.update(
-        _get_rules_file_keys("primary-key", matches=matches, rules_file=rules_file)
-    )
-    replication_keys = _get_catalog_file_keys(
-        "replication-key", matches=matches, catalog_file=raw_catalog_file,
-    )
-    replication_keys.update(
-        _get_rules_file_keys("replication-key", matches=matches, rules_file=rules_file,)
-    )
-    file_text = _make_plan_file_text(
-        matches, primary_keys, replication_keys, excluded_tables
-    )
-    uio.create_text_file(plan_file, file_text)
-    _create_selected_catalog(
-        tap_name,
-        plan_file=plan_file,
-        raw_catalog_file=raw_catalog_file,
-        output_file=selected_catalog_file,
+    _check_rules(
+        tap_name=tap_name,
+        catalog_file=raw_catalog_file,
+        rules_file=rules_file,
+        plan_file_out=plan_file,
+        selected_catalog_file_out=selected_catalog_file,
         replication_strategy=replication_strategy,
-        skip_senseless_validators=SKIP_SENSELESS_VALIDATORS,
+        log_dir=log_dir,
     )
+    if infer_custom_schema:
+        custom_catalog_file = _infer_schema(
+            tap_name,
+            taps_dir,
+            raw_catalog_file=raw_catalog_file,
+            selected_catalog_file=selected_catalog_file,
+            config_file=config_file,
+            catalog_dir=catalog_dir,
+            dockerized=dockerized,
+            tap_exe=tap_exe,
+        )
+        config.push_logs(log_dir, [custom_catalog_file])
+        _check_rules(
+            tap_name=tap_name,
+            catalog_file=custom_catalog_file,
+            rules_file=rules_file,
+            plan_file_out=plan_file,
+            selected_catalog_file_out=selected_catalog_file,
+            replication_strategy=replication_strategy,
+            log_dir=log_dir,
+        )
     _validate_selected_catalog(tap_name, selected_catalog_file=selected_catalog_file)
 
 
@@ -449,13 +631,14 @@ def _make_plan_file_text(
     matches: Dict[str, Dict[str, bool]],
     primary_keys: Dict[str, List[str]],
     replication_keys: Dict[str, List[str]],
-    excluded_tables_list: List[str],
+    table_stream_ids: Dict[str, str],
+    excluded_table_stream_ids: Dict[str, List[str]],
 ) -> str:
     sorted_tables = sorted(matches.keys())
-
     file_text = ""
     file_text += "selected_tables:\n"
     for table in sorted_tables:
+        stream_id = table_stream_ids[table]
         primary_key_cols: List[str] = primary_keys[table]
         replication_key_cols: List[str] = replication_keys[table]
         included_cols = [
@@ -467,6 +650,8 @@ def _make_plan_file_text(
         ]
         ignored_cols = [col for col, selected in matches[table].items() if not selected]
         file_text += f"{' ' * 2}{table}:\n"
+        if table != stream_id:
+            file_text += f"{' ' * 4}stream_id: {stream_id}\n"
         file_text += f"{' ' * 4}primary_key:\n"
         for col in primary_key_cols:
             file_text += f"{' ' * 6}- {col}\n"
@@ -480,10 +665,14 @@ def _make_plan_file_text(
             file_text += f"{' ' * 4}ignored_columns:\n"
             for col in ignored_cols:
                 file_text += f"{' ' * 6}- {col}\n"
-    if excluded_tables_list:
+    if excluded_table_stream_ids:
         file_text += "ignored_tables:\n"
-        for table in sorted(excluded_tables_list):
-            file_text += f"{' ' * 2}- {table}\n"
+        for table, stream_ids in sorted(excluded_table_stream_ids.items()):
+            if len(stream_ids) < 2 and table not in matches.keys():
+                file_text += f"{' ' * 2}- {table}\n"
+            else:
+                for id in stream_ids:
+                    file_text += f'{" " * 2}- {table} ("{id}")\n'
     return file_text
 
 
@@ -505,8 +694,9 @@ def _get_catalog_table_columns(table_object: dict) -> List[str]:
 
 
 def _get_catalog_tables_dict(catalog_file: str) -> dict:
+    """Return a dictionary of streams by their unique stream ID."""
     catalog_full = json.loads(Path(catalog_file).read_text())
-    table_objects = {_get_stream_name(s): s for s in catalog_full["streams"]}
+    table_objects = {_get_stream_id(s): s for s in catalog_full["streams"]}
     return table_objects
 
 
@@ -514,6 +704,10 @@ def _get_stream_name(table_object: dict) -> str:
     return table_object["stream"]
 
 
+def _get_stream_id(table_object: dict) -> str:
+    return table_object.get("tap_stream_id", _get_stream_name(table_object))
+
+
 def _get_table_name(table_object: dict) -> str:
     return table_object.get(
         "table_name", table_object.get("stream", table_object["tap_stream_id"])
@@ -522,7 +716,8 @@ def _get_table_name(table_object: dict) -> str:
 
 @logged(
     "selecting catalog metadata "
-    "from '{tap_name}' source catalog file: {raw_catalog_file}"
+    "from '{tap_name}' source catalog file: {raw_catalog_file}",
+    success_msg=False,
 )
 def _create_selected_catalog(
     tap_name: str,
@@ -533,7 +728,7 @@ def _create_selected_catalog(
     skip_senseless_validators: bool,
 ) -> None:
     taps_dir = config.get_taps_dir()
-    catalog_dir = config.get_catalog_output_dir(tap_name, taps_dir)
+    catalog_dir = config.get_tap_output_dir(tap_name, taps_dir)
     output_file = output_file or os.path.join(catalog_dir, "selected-catalog.json")
     catalog_full = json.loads(Path(raw_catalog_file).read_text())
     plan_file = plan_file or config.get_plan_file(tap_name)
@@ -543,7 +738,10 @@ def _create_selected_catalog(
     included_table_objects = []
     for tbl in sorted(catalog_full["streams"], key=lambda x: _get_stream_name(x)):
         stream_name = _get_stream_name(tbl)
-        if stream_name in plan["selected_tables"].keys():
+        stream_id = _get_stream_id(tbl)
+        if stream_name in plan["selected_tables"].keys() and stream_id == plan[
+            "selected_tables"
+        ][stream_name].get("stream_id", stream_name):
             _set_catalog_file_keys(tbl, plan["selected_tables"][stream_name])
             _select_table(tbl, replication_strategy=replication_strategy)
             for col_name in _get_catalog_table_columns(tbl):
@@ -563,7 +761,8 @@ def _create_selected_catalog(
 
 @logged(
     "validating selected catalog metadata "
-    "from '{tap_name}' selected catalog file: {selected_catalog_file}"
+    "from '{tap_name}' selected catalog file: {selected_catalog_file}",
+    success_msg=False,
 )
 def _validate_selected_catalog(tap_name: str, selected_catalog_file: str,) -> None:
     selected_catalog = json.loads(Path(selected_catalog_file).read_text())
@@ -612,7 +811,7 @@ def _remove_senseless_validators(tbl: dict) -> None:
             "exclusiveMaximum",
             "maxLength",
         ]:
-            if senseless in props:
+            if isinstance(props, dict) and senseless in props:
                 props.pop(senseless)
 
 
@@ -648,7 +847,7 @@ def _create_single_table_catalog(
     output_file: str = None,
 ) -> None:
     taps_dir = config.get_taps_dir()
-    catalog_dir = config.get_catalog_output_dir(tap_name, taps_dir)
+    catalog_dir = config.get_tap_output_dir(tap_name, taps_dir)
     source_catalog_path = full_catalog_file or os.path.join(
         catalog_dir, "catalog-selected.json"
     )
@@ -665,10 +864,13 @@ def _create_single_table_catalog(
         json.dump(catalog_new, f, indent=2)
 
 
-def _table_match_check(match_text: str, select_rules: list) -> bool:
+def _table_match_check(table_name: str, stream_id: str, select_rules: list) -> bool:
     selected = False
     for rule in select_rules:
-        result = _check_table_rule(match_text, rule)
+        if rule.lstrip("!").startswith('"'):
+            result = _check_table_rule(stream_id, rule)
+        else:
+            result = _check_table_rule(table_name, rule)
         if result is True:
             selected = True
         elif result is False:
@@ -676,10 +878,15 @@ def _table_match_check(match_text: str, select_rules: list) -> bool:
     return selected
 
 
-def _col_match_check(match_text: str, select_rules: list) -> bool:
+def _col_match_check(
+    table_name: str, stream_id: str, col_name: str, select_rules: list
+) -> bool:
     selected = False
     for rule in select_rules:
-        result = _check_column_rule(match_text, rule)
+        if rule.lstrip("!").startswith('"'):
+            result = _check_column_rule(f"{stream_id}.{col_name}", rule)
+        else:
+            result = _check_column_rule(f"{table_name}.{col_name}", rule)
         if result is True:
             selected = True
         elif result is False:
@@ -719,9 +926,9 @@ def _check_column_rule(match_text: str, rule_text: str) -> Optional[bool]:
         rule_text = rule_text[1:]
     else:
         match_result = True  # Include if matched
-    table_match = rule_text.split(".")[0]
+    table_rule = rule_text.split(".")[0].lstrip('"').rstrip('"')
     column_match = ".".join(rule_text.split(".")[1:])
-    if not _is_match(match_text.split(".")[0], table_match):
+    if not _is_match(match_text.split(".")[0], table_rule):
         # Non-matching table part; skip!
         return None
     if not _is_match(match_text.split(".")[1], column_match):
@@ -754,7 +961,7 @@ def _check_table_rule(match_text: str, rule_text: str) -> Optional[bool]:
         return None
     if USE_2PART_RULES:
         table_name = match_text.split(".")[0]
-        table_rule = rule_text.split(".")[0]
+        table_rule = rule_text.split(".")[0].lstrip('"').rstrip('"')
     else:  # Using 3-part rules, including tap name as first part
         tap_name = match_text.split(".")[0]
         tap_rule = rule_text.split(".")[0]
@@ -762,6 +969,7 @@ def _check_table_rule(match_text: str, rule_text: str) -> Optional[bool]:
         table_rule = ".".join(rule_text.split(".")[1:])
         if not _is_match(tap_name, tap_rule):
             return None
+    # logging.info(f"Checking for '{table_name}'...")
     if not _is_match(table_name, table_rule):
         return None
     # Table '{match_text}' matched table filter '{table_rule}' in '{rule_text}'"
